"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const mkdirp = require("make-dir");
const path = require("path");
const rimraf = require("rimraf");
const constants = require("../constants");
const data = require("../data");
const utils_1 = require("../utils");
const out = require("../utils/out");
const pkg_1 = require("../utils/pkg");
const versions_1 = require("../utils/versions");
exports.handler = () => {
    const folderPath = utils_1.dealPath(process.argv[3]);
    if (!fs.existsSync(folderPath)) {
        utils_1.exit("Folder Unexist");
    }
    constants.setTargetPath(folderPath);
    const pkgPath = path.resolve(folderPath, "package.json");
    if (!fs.existsSync(pkgPath)) {
        utils_1.exit("Miss `package.json`");
    }
    const sourceProjectVersion = JSON.parse(fs.readFileSync(pkgPath, { encoding: "utf-8" })).yVersion;
    if (!sourceProjectVersion) {
        utils_1.exit("Miss yVersion Param in `package.json`");
    }
    const MAPS = {
        add: {},
        remove: {},
        update: {}
    };
    const versions = versions_1.diffVersions(sourceProjectVersion);
    if (versions.length === 0 && sourceProjectVersion === constants.version) {
        return out.pipe("\n    ::", "This is Latest Project Template");
    }
    versions.forEach((version) => {
        const versionObj = versions_1.getVersionObj(version);
        for (const filePoint of toUniList(versionObj.ADD_LIST)) {
            if (MAPS.remove[filePoint]) {
                MAPS.remove[filePoint] = null;
            }
            MAPS.add[filePoint] = [version];
        }
        for (const filePoint of toUniList(versionObj.UPDATE_LIST)) {
            if (MAPS.add[filePoint]) {
                continue;
            }
            if (!MAPS.update[filePoint]) {
                MAPS.update[filePoint] = [];
            }
            MAPS.update[filePoint].push(version);
        }
        for (const filePoint of toUniList(versionObj.REMOVE_LIST)) {
            if (MAPS.add[filePoint]) {
                MAPS.update[filePoint] = null;
            }
            if (MAPS.update[filePoint]) {
                MAPS.update[filePoint] = null;
            }
            MAPS.remove[filePoint] = [version];
        }
    });
    // Generate Update Queue
    const updateQueue = {};
    for (const t of Object.keys(MAPS)) {
        const map = MAPS[t];
        for (const filePoint of Object.keys(map)) {
            const versionList = map[filePoint];
            for (const version of versionList) {
                if (!updateQueue[version]) {
                    updateQueue[version] = [];
                }
                updateQueue[version].push({
                    filePoint,
                    type: t
                });
            }
        }
    }
    // Update Files
    for (const ver of Object.keys(updateQueue).sort(versions_1.sortFn())) {
        for (const obj of updateQueue[ver]) {
            const { filePoint } = obj;
            switch (obj.type) {
                case "add":
                    add(filePoint);
                    break;
                case "update":
                    const versionObj = versions_1.getVersionObj(ver);
                    if (!fs.existsSync(path.resolve(constants.targetPath, filePoint))) {
                        if (!versionObj.isIgnoreCheck(filePoint)) {
                            utils_1.exit(`Miss \`${filePoint}\``);
                        }
                        continue;
                    }
                    versionObj.update(filePoint.replace(/^\.\//, ""));
                    out.pipe("UPDATE", filePoint);
                    break;
                case "remove":
                    remove(filePoint);
                    break;
            }
        }
    }
    // Update `package.json`
    new pkg_1.Pkg(constants.targetPath)
        .modify((obj) => {
        obj.yVersion = constants.version;
        return obj;
    })
        .save();
    out.pipe("\n    ::", `v${sourceProjectVersion} => v${constants.version} Updated`);
    return true;
};
const add = (filePoint) => {
    const rawFileName = data.files[filePoint];
    const from = path.resolve(constants.resourcesPath, rawFileName);
    const to = path.resolve(constants.targetPath, filePoint);
    if (!fs.existsSync(path.dirname(to))) {
        mkdirp.sync(path.dirname(to));
    }
    if (fs.existsSync(to)) {
        out.pipe("SKIP", filePoint);
    }
    else {
        fs.createReadStream(from).pipe(fs.createWriteStream(to));
        out.pipe("CREATE", filePoint);
    }
};
const remove = (filePoint) => {
    const filePath = path.resolve(constants.targetPath, filePoint);
    rimraf.sync(filePath);
    out.pipe("DELETE", filePoint);
};
const toUniList = (list = []) => {
    return list.map((item) => {
        return ("./" +
            path
                .resolve(__dirname, item)
                .replace(__dirname, "")
                .replace(/\\/g, "/")
                .replace(/^./, ""));
    });
};
