"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ftconfig = require("ftconfig");
const path = require("path");
class Pkg {
    constructor(p) {
        const filepath = path.resolve(p.toString(), "package.json");
        this.config = ftconfig.readFile(filepath, { type: "json" });
    }
    save() {
        this.config.save();
        return this;
    }
    modify(fn) {
        this.config.modify(fn);
        return this;
    }
    toObject() {
        return this.config.toObject();
    }
    getSaveDependency(name) {
        return this.toObject().dependencies[name];
    }
    updateSaveDependency(name, version) {
        this.config.modify((obj) => {
            obj.dependencies[name] = version;
            return obj;
        });
        return this;
    }
    deleteSaveDependency(name) {
        return this.deleteSaveDependencies(name);
    }
    deleteSaveDependencies(...names) {
        this.config.modify((object) => {
            names.forEach((name) => {
                delete object.dependencies[name];
            });
            return object;
        });
        return this;
    }
    getDevDependency(name) {
        return this.toObject().devDependencies[name];
    }
    updateDevDependency(name, version) {
        this.config.modify((object) => {
            object.devDependencies[name] = version;
            return object;
        });
        return this;
    }
    deleteDevDependency(name) {
        return this.deleteDevDependencies(name);
    }
    deleteDevDependencies(...names) {
        this.config.modify((object) => {
            names.forEach((name) => {
                delete object.devDependencies[name];
            });
            return object;
        });
        return this;
    }
    getScript(action) {
        return this.toObject().scripts[action];
    }
    updateScript(action, command) {
        this.config.modify((object) => {
            if (command) {
                object.scripts[action] = command;
            }
            else {
                delete object.scripts[action];
            }
            return object;
        });
        return this;
    }
    deleteScript(action) {
        return this.deleteScript(action);
    }
    deleteScripts(...actions) {
        this.config.modify((object) => {
            actions.forEach((action) => {
                delete object.scripts[action];
            });
            return object;
        });
        return this;
    }
}
exports.Pkg = Pkg;
