"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const constants = require("../constants");
/**
 * 和版本相关的方法
 */
/** */
let historyVersions = [];
/**
 * 获取已有的更新列表
 */
exports.getHistoryVersions = () => {
    if (historyVersions.length === 0) {
        historyVersions = fs
            .readdirSync(path.resolve(__dirname, "../patches"))
            .filter((name) => /\d+\.\d+\.\d+\.[jt]s$/.test(name))
            .map((name) => name.replace(/\.[jt]s$/, ""));
    }
    return historyVersions;
};
/**
 * Parse 版本号
 */
exports.parseVersion = (ver) => {
    const matches = ver.match(/(\d+)\.(\d+)\.(\d+)/);
    const verObj = {
        major: parseInt(matches[1], 10),
        minor: parseInt(matches[2], 10),
        patch: parseInt(matches[3], 10)
    };
    return Object.assign({
        toString: () => {
            return `${verObj.major}.${verObj.minor}.${verObj.patch}`;
        }
    }, verObj);
};
exports.getVersionObj = (ver) => {
    const obj = require(`../patches/${ver}`);
    return Object.assign({
        isIgnoreCheck: (filePoint) => {
            if (!obj.IGNORE_CHECK_LIST) {
                return false;
            }
            return (Array.from(obj.IGNORE_CHECK_LIST)
                .map((item) => path.resolve(constants.targetPath, item))
                .indexOf(path.resolve(constants.targetPath, filePoint)) !== -1);
        }
    }, obj);
};
exports.diffVersions = (ver) => {
    if (ver === constants.version) {
        return [];
    }
    const curVersion = exports.parseVersion(ver);
    return exports.getHistoryVersions()
        .reduce((arr, v) => {
        arr.push(exports.parseVersion(v));
        return arr;
    }, [])
        .sort(exports.sortFn())
        .filter(exports.filterFn(curVersion))
        .map((v) => v.toString());
};
exports.sortFn = (asc = true) => {
    return (a, b) => {
        const aa = typeof a === "string" ? exports.parseVersion(a) : a;
        const bb = typeof b === "string" ? exports.parseVersion(b) : b;
        for (const level of ["major", "minor", "patch"]) {
            if (aa[level] === bb[level]) {
                continue;
            }
            return asc ? aa[level] - bb[level] : bb[level] - aa[level];
        }
        return 0;
    };
};
exports.filterFn = (baseVersion) => {
    return (v) => {
        for (const level of ["major", "minor", "patch"]) {
            if (v[level] > baseVersion[level]) {
                return true;
            }
            if (v[level] < baseVersion[level]) {
                return false;
            }
        }
        return false;
    };
};
